﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using Thumbnails.Data;

namespace Thumbnails
{
	public partial class Page : UserControl
	{
    private FrameworkElement _expandedMedia = null;

    public Page()
    {
      // Required to initialize variables
      InitializeComponent();

      FrameworkElement media = null;
      int index = 0;
      do
      {
        media = this.FindName("Media" + index++) as FrameworkElement;
        if (media != null)
        {
          media.Cursor = Cursors.Hand;
          media.MouseLeftButtonDown
            += new MouseButtonEventHandler(media_MouseLeftButtonDown);
        }
      }
      while (media != null);

      DataFile dataFile = new DataFile();
      User user = dataFile.LoadUser();
      if (user.LastVisit == null)
      {
        LastVisitTextBlock.Text = "(This is your first visit)";
      }
      else
      {
        LastVisitTextBlock.Text = "(Your last visit: "
          + user.LastVisit.Value.ToShortDateString()
          + " " + user.LastVisit.Value.ToLongTimeString() + ")";
      }
      user.SetLastVisit();
      dataFile.SaveUser(user);
    }

    void media_MouseLeftButtonDown(object sender,
      MouseButtonEventArgs e)
    {
      FrameworkElement castedSender = sender as FrameworkElement;

      // If needed, pause movie
      if (_expandedMedia != null)
      {
        if (_expandedMedia is MediaElement)
        {
          (_expandedMedia as MediaElement).Pause();
        }
        // Hide currently displayed media
        Display.Fill = new SolidColorBrush(Colors.Transparent);
        FrameworkElement currentMedia = _expandedMedia;
        _expandedMedia = null;

        if (castedSender == currentMedia)
        {
          return;
        }
      }

      // Set size and position of DisplayRectangle
      double sizeFactor
        = castedSender.ActualWidth / castedSender.ActualHeight;
      Display.Width = Display.ActualHeight * sizeFactor;
      double sideMargin
        = (DisplayBackground.ActualWidth - Display.Width) / 2;
      Display.Margin
        = new Thickness(sideMargin, 10, sideMargin, 10);

      // Set the Display and start movie (if needed)
      if (castedSender is MediaElement)
      {
        VideoBrush brush = new VideoBrush();
        brush.SetSource(castedSender as MediaElement);
        (castedSender as MediaElement).Play();
        Display.Fill = brush;
      }
      if (castedSender is Image)
      {
        ImageBrush brush = new ImageBrush();
        BitmapImage source
          = (castedSender as Image).Source as BitmapImage;
        brush.ImageSource
          = new BitmapImage(source.UriSource);
        Display.Fill = brush;
      }

      _expandedMedia = castedSender;
    }

    private void MediaElement_MediaEnded(object sender, RoutedEventArgs e)
    {
      (sender as MediaElement).Stop();
      (sender as MediaElement).Play();
    }
  }
}