﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO;
using System.IO.IsolatedStorage;
using System.Collections.Generic;

namespace Thumbnails.Data
{
  public class DataFile
  {
    private const string DATA_FILE_NAME = "ThumbnailsData.2.txt";

    private Dictionary<string, User> _users;

    internal void LoadUsers()
    {
      _users = new Dictionary<string, User>();

      using (IsolatedStorageFile store
        = IsolatedStorageFile.GetUserStoreForApplication())
      {
        if (store.FileExists(DATA_FILE_NAME))
        {
          using (IsolatedStorageFileStream streamRead
            = store.OpenFile(DATA_FILE_NAME, FileMode.Open))
          {
            using (StreamReader reader = new StreamReader(streamRead))
            {
              string line;
              do
              {
                line = reader.ReadLine();
                if (line != null)
                {
                  User newUser = new User(line);
                  _users.Add(newUser.Name, newUser);
                }
              }
              while (line != null);
            }
          }
        }
      }
    }

    internal void SaveUsers()
    {
      using (IsolatedStorageFile store
        = IsolatedStorageFile.GetUserStoreForApplication())
      {
        using (IsolatedStorageFileStream streamWrite
          = store.OpenFile(DATA_FILE_NAME, FileMode.OpenOrCreate))
        {
          using (StreamWriter writer = new StreamWriter(streamWrite))
          {
            foreach (User user in _users.Values)
            {
              writer.WriteLine(user.ToString());
            }
          }
        }
      }
    }

    internal void AddUser(User newUser)
    {
      if (newUser.Name == null
        || newUser.Name.Length == 0)
      {
        throw new Exception("Name may not be empty");
      }
      if (_users.ContainsKey(newUser.Name))
      {
        throw new Exception("This name already exists");
      }
      _users.Add(newUser.Name, newUser);
      SaveUsers();
    }

    internal bool ValidateUser(User inputUser)
    {
      if (inputUser == null)
      {
        // Refuse to validate null users
        return false;
      }
      User existingUser = GetUser(inputUser.Name);
      return (existingUser == inputUser);
    }

    internal User GetUser(string name)
    {
      if (_users.ContainsKey(name))
      {
        return _users[name];
      }
      else
      {
        return null;
      }
    }
  }
}
