﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using Thumbnails.Data;
using System.Windows.Browser;
using System.Net;
using Thumbnails.UserManagement;

namespace Thumbnails
{
	public partial class Page : UserControl
	{
    [ScriptableMember]
    public event EventHandler LoginClicked;

    private FrameworkElement _expandedMedia = null;
    private double _expansionFactor = 1.0;
    private Storyboard _fadeStoryboard = null;
    private MediaInformationFile _mediaFile;
    private UserManagementServiceClient _serviceClient
      = new UserManagementServiceClient();

    private int _flickrIndex = 1;

    public Page()
    {
      // Required to initialize variables
      InitializeComponent();

      Storyboard expandStoryboard
        = TopUserControl.Resources["ExpandStoryboard"] as Storyboard;
      expandStoryboard.Completed
        += new EventHandler(expandStoryboard_Completed);

      HtmlPage.RegisterScriptableObject("Page", this);

      ThumbnailsViewer.SelectionChanged
        += new SelectionChangedEventHandler(ThumbnailsViewer_SelectionChanged);

      this.Loaded += new RoutedEventHandler(Page_Loaded);

      _serviceClient.AddUserCompleted
        += new EventHandler<AddUserCompletedEventArgs>(
          _serviceClient_AddUserCompleted);
      _serviceClient.ValidateUserCompleted
        += new EventHandler<ValidateUserCompletedEventArgs>(
          _serviceClient_ValidateUserCompleted);
    }

    void Page_Loaded(object sender, RoutedEventArgs e)
    {
      _mediaFile = new MediaInformationFile();
      _mediaFile.Loaded += new EventHandler(_mediaFile_Loaded);
      _mediaFile.DownloadProgressChanged
        += new DownloadProgressChangedEventHandler(
          _mediaFile_DownloadProgressChanged);

      _mediaFile.AnyProgressChanged
        += new EventHandler(_mediaFile_AnyProgressChanged);
      _mediaFile.NewMediaFound
        += new EventHandler(_mediaFile_NewMediaFound);
      
      _mediaFile.LoadMediaFile();
    }

    void _mediaFile_AnyProgressChanged(object sender, EventArgs e)
    {
      AnyProgressChangedEventArgs args = e as AnyProgressChangedEventArgs;
      LiveStatustextBlock.Text = args.Message;
    }
    void _mediaFile_NewMediaFound(object sender, EventArgs e)
    {
      NewMediaFoundEventArgs args = e as NewMediaFoundEventArgs;
      MediaExCollection itemsSource = ThumbnailsViewer.ItemsSource;
      itemsSource.Add(args.MediaInfo);
      LiveStatustextBlock.Text
        = string.Format("New Flickr picture found ({0})",
          _flickrIndex++);
    }

    void _mediaFile_DownloadProgressChanged(object sender,
      DownloadProgressChangedEventArgs e)
    {
      StatusTextBlock.Text = string.Format("Loading {0}% ({1} / {2})",
        e.ProgressPercentage, e.BytesReceived, e.TotalBytesToReceive);
    }

    void _mediaFile_Loaded(object sender, EventArgs e)
    {
      MediaFileLoadedEventArgs args = e as MediaFileLoadedEventArgs;

      if (args.Collection == null)
      {
        StatusTextBlock.Text = "There was an error";
        if (args.Error != null)
        {
          ErrorTextBlock.Text = args.Error.Message
            + Environment.NewLine + args.Error.StackTrace;
        }
        return;
      }

      MediaExCollection mediaCollection
        = Resources["MediaDataSource"] as MediaExCollection;

      bool foundBrush = false;
      foreach (MediaEx mediaInfo in args.Collection)
      {
        mediaCollection.Add(mediaInfo);

        if (!foundBrush
          && mediaInfo.Type == MediaInfo.MediaType.Image)
        {
          ImageBrush brush = new ImageBrush();
          brush.ImageSource = mediaInfo.ImageSource;
          TitleTextBlock1.Foreground = brush;
          TitleTextBlock2.Foreground = brush;
          foundBrush = true;
        }
      }

      Storyboard cacheFadeOut
        = Resources["CacheFadeOutStoryboard"] as Storyboard;
      cacheFadeOut.Completed
        += new EventHandler(cacheFadeOut_Completed);
      cacheFadeOut.Begin();
    }

    void cacheFadeOut_Completed(object sender, EventArgs e)
    {
      Cache.Visibility = Visibility.Collapsed;
    }

    void ThumbnailsViewer_SelectionChanged(object sender,
      SelectionChangedEventArgs e)
    {
      if (e.AddedItems.Count > 0)
      {
        ExpandCollapseMedia(e.AddedItems[0] as FrameworkElement);
      }
    }

    void expandStoryboard_Completed(object sender, EventArgs e)
    {
      Canvas.SetLeft(Display, Canvas.GetLeft(ThumbDisplay1));
      Display.Width = ThumbDisplay1.ActualWidth * _expansionFactor;
      Display.Fill = ThumbDisplay1.Fill;
      Display.Opacity = 1;
      if (_fadeStoryboard != null)
      {
        _fadeStoryboard.Stop();
        _fadeStoryboard = null;
      }
      ThumbDisplay1.Opacity = 0;
      ThumbDisplay1.Fill = null;
      (sender as Storyboard).Stop();
    }

    void ExpandCollapseMedia(FrameworkElement media)
    {
      // If needed, pause movie and fade display
      if (_expandedMedia != null)
      {
        if (_expandedMedia is MediaElement)
        {
          (_expandedMedia as MediaElement).Pause();
        }
      
        FrameworkElement currentMedia = _expandedMedia;
        _expandedMedia = null;
      
        _fadeStoryboard = new Storyboard();
        _fadeStoryboard.Completed += new EventHandler(_fadeStoryboard_Completed);
      
        DoubleAnimation anim = new DoubleAnimation();
        Storyboard.SetTarget(anim, Display);
        Storyboard.SetTargetProperty(anim, new PropertyPath("Opacity"));
        anim.Duration = new Duration(TimeSpan.FromSeconds(0.9));
        anim.To = 0;
        _fadeStoryboard.Children.Add(anim);
      
        _fadeStoryboard.Begin();
      
        if (media == currentMedia)
        {
          return;
        }
      }

      // Set size and position of the displayed media
      double sizeFactor
        = media.ActualWidth / media.ActualHeight;
      double newHeight = DisplayBackground.ActualHeight;
      double newWidth = newHeight * sizeFactor;
      double sideMargin
        = (DisplayBackground.ActualWidth - newWidth) / 2;
      _expansionFactor
        = DisplayBackground.ActualHeight / media.ActualHeight;

      if (sideMargin < 0)
      {
        newWidth = DisplayBackground.ActualWidth;
        newHeight = newWidth / sizeFactor;
        sideMargin = 0;
        _expansionFactor
          = DisplayBackground.ActualWidth / media.ActualWidth;
      }

      // Set the placeholder's size and calculate its proportions
      ThumbDisplay1.Width = media.ActualWidth;
      ThumbDisplay1.Height = media.ActualHeight;

      // Getting the coordinates of the clicked thumbnail
      GeneralTransform gtStart =
        media.TransformToVisual(AnimationCanvas);
      Point offsetStart = gtStart.Transform(new Point(0, 0));
      ThumbDisplay1.SetValue(Canvas.LeftProperty, offsetStart.X);
      ThumbDisplay1.SetValue(Canvas.TopProperty, offsetStart.Y);

      // Getting the coordinates of the DisplayBackground rectangle
      GeneralTransform gtDisplay =
        DisplayBackground.TransformToVisual(AnimationCanvas);
      Point offsetDisplay = gtDisplay.Transform(new Point(0, 0));

      // Get the storyboard and set the animations' parameters
      Storyboard expandStoryboard
        = TopUserControl.Resources["ExpandStoryboard"] as Storyboard;

      foreach (Timeline child in expandStoryboard.Children)
      {
        string property = Storyboard.GetTargetProperty(child).Path;
        if (property.IndexOf("TranslateTransform.X") > -1)
        {
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[0].Value = 0;
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[1].Value
            = offsetDisplay.X - offsetStart.X + sideMargin;
        }
        if (property.IndexOf("TranslateTransform.Y") > -1)
        {
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[0].Value = 0;
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[1].Value
            = offsetDisplay.Y - offsetStart.Y;
        }
        if (property.IndexOf("ScaleTransform.ScaleX") > -1)
        {
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[0].Value = 1;
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[1].Value
            = _expansionFactor;
        }
        if (property.IndexOf("ScaleTransform.ScaleY") > -1)
        {
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[0].Value = 1;
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[1].Value
            = _expansionFactor;
        }
      }
      
      // Set the Display and start movie (if needed)
      if (media is MediaElement)
      {
        VideoBrush brush = new VideoBrush();
        brush.SetSource(media as MediaElement);
        (media as MediaElement).Play();
        ThumbDisplay1.Fill = brush;
      }
      if (media is Image)
      {
        ImageBrush brush = new ImageBrush();
        MediaEx context = (media as Image).DataContext as MediaEx;
        brush.ImageSource = context.ImageSource;
        ThumbDisplay1.Fill = brush;
      }

      _expandedMedia = media;
      //Start the Storyboard
      expandStoryboard.Begin();
    }

    void _fadeStoryboard_Completed(object sender, EventArgs e)
    {
      Display.Fill = null;
      (sender as Storyboard).Stop();
    }

    [ScriptableMember]
    public string AddUser(string name, string password)
    {
      _serviceClient.AddUserAsync(name, password);
      return null; // success
    }
    [ScriptableMember]
    public string ValidateUser(string name, string password)
    {
      _serviceClient.ValidateUserAsync(name, password);
      return null;
    }

    void _serviceClient_ValidateUserCompleted(object sender,
      ValidateUserCompletedEventArgs e)
    {
      UserInformation userInfo = e.Result;
      if (userInfo.PasswordOk)
      {
        LastVisitTextBlock.Text = "(Your last visit was: "
          + userInfo.LastVisit.Value.ToShortDateString()
          + " " + userInfo.LastVisit.Value.ToLongTimeString() + ")";
      }
      else
      {
        LastVisitTextBlock.Text = "";
        HtmlPage.Window.Alert("Invalid user/password combination");
        // Here you should probably stop the application!
      }
    }

    void _serviceClient_AddUserCompleted(object sender,
      AddUserCompletedEventArgs e)
    {
      if (e.Result != null)
      {
        HtmlPage.Window.Alert(e.Result);
      }
      else
      {
        LastVisitTextBlock.Text = "(This is your first visit)";
      }
    }

    public void OnLoginClicked(EventArgs e)
    {
      if (LoginClicked != null)
      {
        LoginClicked(this, e);
      }
    }

    private void LoginLink_Click(object sender, RoutedEventArgs e)
    {
      EventArgs args = new EventArgs();
      OnLoginClicked(args);
    }
  }
}